/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Don Jul 18 14:31:28 CEST 2002
    copyright            : (C) 2002 by Michael Majdic
    email                : michael.majdic@gmx.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <fstream.h>
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#define MODE0 0
#define MODE1 1
#define MODE2 2
#define FALSE 0
#define TRUE  1
#define MAXFILELENGTH 256
#define MAXLINELENGTH 3004
#define MAXMAPPEDVECTORS 10000
#define MAXPHRASELENGTH 1024
#define MAXPHRASESPERUNIT 1000
#define KEACALL "java KEAKeyphraseExtractor"


/* globals */
// default options
int mode =MODE1;
char model[256] = "kea_model";
int number_of_phrases =15;
char path[256] = "output/";
char keypath[12] ="output/key/";
char keyfileending[5] =".key";
char sourcepath[64] ="output/files/";
int labelfiles_exist = 1;
char kea_tmp_path[MAXFILELENGTH] = "output/tmp_kea/";
char labelfiles_path[MAXFILELENGTH] = "output/labels/";

struct files
{
	char name[MAXFILELENGTH];
};

struct unit
{
	int pos_x;
	char pos_x_str[256];
	int pos_y;
	char pos_y_str[256];
	char node_id[256];
	char quanterror_node[64];
	int nr_mapped_vecs;
	files filelist[MAXMAPPEDVECTORS];
	int nr_soms_mapped;
	files url_mapped_soms;
};

struct phrase
{
	int wc_phrase;
	char phrase[MAXPHRASELENGTH];
	char stemmed_phrase[MAXPHRASELENGTH];
	double score;
};

// prototype

int do_the_parsman(char map[MAXFILELENGTH]);
int do_the_catman(unit curr_unit,FILE *opf, char map[MAXFILELENGTH]);
int do_the_calcman(unit curr_unit,FILE *opf);
int phrase_exists(phrase extracted_phrases[MAXPHRASESPERUNIT], phrase current_phrase, int dim);
int write_map_keyfile(char map[MAXFILELENGTH]);
void sort_array(phrase phrases[MAXPHRASESPERUNIT], int phrasecount);
int is_splitted_file(char lastfilename[MAXFILELENGTH],char newfilename[MAXFILELENGTH]);
int filter_phrases (phrase phrases[MAXPHRASESPERUNIT], int phrasecount);
void dispose_phrase(phrase phrases[MAXPHRASESPERUNIT],int position, int phrasecount);
int compare_phrases(phrase phrase1, phrase phrase2);
int sum_phrases(phrase phrases[MAXPHRASESPERUNIT], int phrasecount, phrase current_phrase);
// int write_labels(FILE *input, FILE *output);


// ######################################################
// Function Main
// ######################################################

int main(int argc, char *argv[])
{
	FILE *propfile;
	FILE *processfile;
	char line[MAXFILELENGTH];
	char map[MAXFILELENGTH];
	char *token;
	char *substring;
	char help_string[256];

	if(argc != 3)
	{
		fprintf(stderr,"Usage: %s processfile config_file\n", argv[0]);
		return 0;
	}

// read and parse the optionfile expected as the second command lien argument	

	propfile = fopen(argv[2],"r");
	while (fgets(line, MAXFILELENGTH, propfile) != NULL)
	{
  	token=strtok(line, " ");
 		while ((substring=strtok(NULL, " ")) != NULL)
		{
			if (strcmp(token, "MODE")==0)
    	{  // either mode0 mode1 or mode2 i.e. 0, 1 or 2 is expected
 				strcpy(help_string,substring);
    		mode = atol(help_string);	
				if ((mode != MODE0) && (mode != MODE1) && (mode != MODE2))
				{
					fprintf(stderr,"Invalid value for option MODE in given option file, ciao ...\n");
					return 0;
				}
    	}
    	else if (strcmp(token, "KEA_MODEL")==0)
    	{  // the filename of the kea_model is expected
    		strcpy(model,substring);
				model[strlen(model)-1] = '\0';

    	}
   		else if (strcmp(token, "NUMBEROFPHRASES")==0)
    	{  // how many phrases should be extracted / used
    		strcpy(help_string,substring);
    		number_of_phrases = atol(help_string);	
    	}
    	else if (strcmp(token, "OUTPUTPATH")==0)
    	{  // the path where the unitfiles are expected and the modiefied file are written
    		strcpy(path,substring);
				path[strlen(path)-1] = '\0';
    	}
   		else if (strcmp(token, "KEYPATH")==0)
    	{  // the path where the 'keyfiles' are expected, i.e. the file generated by KEA
				strcpy(keypath,substring);
				keypath[strlen(keypath)-1] = '\0';
    	}
   		else if (strcmp(token, "KEAFILEEXTENSION")==0)
    	{  // the EXTENSION of the kea files, normally is should be .key
				strcpy(keyfileending,substring);
				keyfileending[strlen(keyfileending)-1] = '\0';
    	}
  		else if (strcmp(token, "SOURCEPATH")==0)
    	{  // the path, where the orginal sourcefiles can be found
				strcpy(sourcepath,substring);
				sourcepath[strlen(sourcepath)-1] = '\0';
    	}
  		else if (strcmp(token, "TMPKEAPATH")==0)
    	{  // a temporary path
				strcpy(kea_tmp_path,substring);
				kea_tmp_path[strlen(kea_tmp_path)-1] = '\0';
    	}
		}
	}
  fclose(propfile);

//close optionfile and open the generated processfile

	processfile = fopen(argv[1],"r");
	while (fgets(line, MAXFILELENGTH, processfile) != NULL)
	{
  	memset(map,0,strlen(map) * sizeof(char));
  	strcpy(map, path);
  	strncat(map,line,strlen(line)-1);
		if (do_the_parsman(map)==0)
		{
			fprintf(stderr, "Error parsing unit file for map %s\n", map);
 			return 0;
 		}
		if (write_map_keyfile(map)==0)
 		{
 			//fprintf(stderr, "Error writing keyfile for map %s\n", map);
 			return 0;
 		} 		
  	
	}
  fclose(processfile);
}
	
	

/* #############################################
int (do_the_parsman(char map[MAXFILELENTH]
#############################################
Parses the given file map and writes the output to map.mmx
###########################################
*/


int do_the_parsman(char map[MAXFILELENGTH])
{
  FILE *fp;
  FILE *ofp;
  FILE *mappedsom_keyfile;
	FILE *mapped_labelunit;
//  FILE *labelfile;
  char line[MAXLINELENGTH];
  char help_line[MAXLINELENGTH];
	char keyline[MAXLINELENGTH];
  char *substring;
  char *token;
  char *token2;
  int mid_of_word;
	int endofline;
	int go_on;
  char help_mapped_som[MAXFILELENGTH];
//	char help_labelfilename[MAXFILELENGTH];
//	char help_labelfilename2[MAXFILELENGTH];
//	char labelfilename[MAXFILELENGTH];
	unit current_unit;
  char help_string[256];
  int j;
  char outputfilename[MAXFILELENGTH];
  char mapped_som[MAXFILELENGTH];
//  char c;
	char unitfile[MAXFILELENGTH];
	char newfilename[MAXFILELENGTH];

  /* Try to open Unitfile */


	if ((fp=fopen(map,"r"))==0)
  {
  	/* TODO: Clean up */
  	fprintf(stderr, "Unitfile %s not found....\n", unitfile);
  	return 0;
  }

	strcpy(outputfilename, map);
	strcat(outputfilename,  ".labelunit"); // write to new unit file .labelunit
	fprintf(stdout, "Processing %s and writing %s ...\n", map,outputfilename);
  if ((ofp=fopen(outputfilename, "w"))==0)
  {
  	/* TODO: Clean up */
  	fprintf(stderr, "Error writing file\n");
  	return 0;
  }

//  // Open the according label file if option labelfiles_exist = TRUE
//  if (labelfiles_exist == 1)
//  {
//		strcpy(help_labelfilename, strrchr(map,'/')+1); //extract the filename from path
//		strncpy(help_labelfilename2, help_labelfilename, strlen(help_labelfilename) -4); //remove ending unit
//   	strcpy(labelfilename, labelfiles_path); // copa labelspath
//   	strcat(labelfilenachar unitlabelname[MAXFILELENGTH]me, help_labelfilename2); //cat filename
//   	strcat(labelfilename, "html.labels");// and the correct ending
//		if ((labelfile=fopen(labelfilename, "r"))==0)
//  	{
//  		fprintf(stderr, "Error opening labelfile %s\n", labelfilename);
//  		return 0;
//  	}
//  }


	// Begin parsing, line by line till EOF reached
//	line = (char *) malloc(1 * sizeof(char));

  while ((fgets(line,MAXLINELENGTH,fp))!=NULL)
  {
  	fprintf (ofp,"%s", line);
  	// check if line is to long (is just handled if token is MAPPED_VECS)
  	strcpy(help_line, line);
  	if (line[(strlen(line)-1)] != '\n')
		{ 			// lines to long
			
//			fprintf(stdout, "LIne1 %s\n", line);
			endofline = FALSE;
      //c = line[(strlen(line)-1)];
			if (line[(strlen(line)-1)] != ' ')
		  { // last character is a blank, so we are not in the middle of a word
 				//fprintf(stdout,"2char:%c:\n",c);
		  	mid_of_word=FALSE;
		  }
		  else
		  {
		  	mid_of_word=TRUE;
		  }
		}
		else
		{ // line is not too long
			endofline = TRUE;
		}

 		token=strtok(line, " ");
 		while ((substring=strtok(NULL, " ")) != NULL)
   	{
			//fprintf(stderr,"token: %s ::", token);
			//fprintf(stderr,"dahinta %s\n", substring);
			
  		if (strcmp(token, "$POS_X")==0)
    	{  // new unit because X-Pos is given
 				strcpy(current_unit.pos_x_str,substring);
    		current_unit.pos_x = atol(current_unit.pos_x_str);
    	}
    	else if (strcmp(token, "$POS_Y")==0)
    	{  // Y-Position of the unit
    		strcpy(current_unit.pos_y_str,substring);
    		current_unit.pos_y = atol(current_unit.pos_y_str);
    	}
   		else if (strcmp(token, "$NODE_ID")==0)
    	{  // Node_id
    		strcpy(current_unit.node_id,substring);
    	}
    	else if (strcmp(token, "$QUANTERROR_NODE")==0)
    	{  // QUANTERROR_Node
    		strcpy(current_unit.quanterror_node,substring);
    	}
   		else if (strcmp(token, "$NR_VEC_MAPPED")==0)
    	{
   	 	//printf("%s\n",token);
    		strcpy(help_string,substring);
    		current_unit.nr_mapped_vecs = atol(help_string);
    		//printf("%d\n",current_unit.nr_mapped_vecs);
    	}
    	else if (strcmp(token, "$MAPPED_VECS")==0)
    	{  // read filesname into current_unit.filelist
  			go_on = TRUE;
  			j=0;
				// handle line that are longe than MAXLINELENGTH
				while (go_on == TRUE)     	
				{
    			token2=strtok(help_line, " ");
    			if ((strcmp(token2, "$MAPPED_VECS")) != 0)
    			{
    				
    				strcpy(newfilename,token2);
    				if (is_splitted_file(current_unit.filelist[j-1].name,newfilename) != 0)
    				{
    					strcat(current_unit.filelist[j-1].name,token2);
    				}
    				else
    				{    		
			 				if (strcmp(token2, "\n") != 0)
 				 			{
 				 				// otherwise add a new file
 				 				strcpy(current_unit.filelist[j].name, token2);
 				 				j++;
 				 			}	   	
			 			}
			 		}
    			while ( (substring=strtok(NULL, " ")) != NULL)
    	   	{
    	   		if (strcmp(substring, "\n") != 0)
 				 		{
 				 			strcpy(current_unit.filelist[j].name, substring);
 				 			j++;
 				 		}	   	
  			  }
  				if (endofline == FALSE)
  				{
  					fgets(line, MAXLINELENGTH, fp);
  					fprintf (ofp,"%s", line);
						strcpy(help_line, line);
						//c = line[(strlen(line)-1)];	
						if ((line[(strlen(line)-1)])!='\n')
  					{ 			// lines to long
							endofline = FALSE;
							if (line[(strlen(line)-1)] == ' ')
		  				{ // last character is a blank, so we are not in the middle of a word
				 				//fprintf(stdout,"2char:%c:\n",c);
		  					mid_of_word=FALSE;
		  				}
						  else
		  				{
		  					mid_of_word=TRUE;
		  				}
		  			}
						else
						{ // line is not too long
							endofline = TRUE;
						}
    			}
    			else
    			{
    				go_on=FALSE;
    			}		
  			}
 				for (int k=0;k < current_unit.nr_mapped_vecs;k++)
				{
					fprintf(ofp,"$VEC %s\n",current_unit.filelist[k].name);
				}
				  			if (j != current_unit.nr_mapped_vecs)
	 			{
 					fprintf(stderr, "j= %d , mapped_vecs = %d\n",j,current_unit.nr_mapped_vecs);
 					fprintf(stderr,"Corrupt Unit-GHSomfile %s..., ciao\n",map);
 					return 0;
 				}

 				// if lablelfiles provided call write_labels
// 				if (labelfiles_exist == 1)
// 				{
//					if (write_labels(labelfile,ofp)==0)
// 					{
// 						fprintf(stderr,"returning  from labelfile %s\n",labelfilename);
// 						return 0;
// 					}
//				}
 				if (mode == MODE0 || mode == MODE2)
 				// cat all files (vectors) in one single file, do a keyphraseextraction and write the
 				// best X (=option) keywords with scoring in the section mode0
  			{
 					if (!(do_the_catman(current_unit,ofp, map)))
 					{
 						fprintf(stderr, "AN Error occured while catting the files and parsing them\n");
 						return 0;
 					}
 				}
 			}
    	else if (strcmp(token, "$NR_SOMS_MAPPED")==0)
   		{  // nr soms mapped
				strcpy(help_string,substring);
    		current_unit.nr_soms_mapped = (int)atol(help_string);
    		if (current_unit.nr_soms_mapped == 0 && ((mode==MODE1)||(mode==MODE2)))
    		{
	    		if(do_the_calcman(current_unit,ofp)==0)
	    		{
	    			fprintf(stderr,"Error calculating keyphrases for Unit %d %d in map %s\n",current_unit.pos_x,current_unit.pos_y,map);
	    			return 0;	
	    		}
    			
    		}
    	}
   		else if (strcmp(token, "$URL_MAPPED_SOMS")==0)
   		{  //  url mapped soms
   			memset(mapped_som, 0 , sizeof(mapped_som));
   			strcpy(current_unit.url_mapped_soms.name,substring);
   			strcpy(help_mapped_som, strrchr(substring, '/')+1); //extract the filename from path
   			strcpy(mapped_som,keypath);
   			strncat(mapped_som, help_mapped_som, strlen(help_mapped_som) -4);
   			strcat(mapped_som, "unit"); // string operations in c is boring shit
   			strcat(mapped_som,keyfileending);
   			//fprintf(stdout,"map: %s mapped_som: %s\n",current_unit.url_mapped_soms.name, mapped_som);
  			if ((mappedsom_keyfile=fopen(mapped_som,"r"))==NULL)
  			{
  				fprintf(stderr,"Error opening keyfile %s\n",mapped_som);
  				return 0;
  			}
  			while (fgets(keyline,MAXLINELENGTH,mappedsom_keyfile) != NULL)
  			{
  				fprintf(ofp,"%s",keyline);
  			}
  			fclose(mappedsom_keyfile);
				// 10.1.2001 :
				// in das dartunterliegende *labelunit file wird der name der darberliegenden
				// SOM hineingeschrieben
				//
				memset(mapped_som, 0 , sizeof(mapped_som));
				strcpy(help_mapped_som, current_unit.url_mapped_soms.name);
				strncat(mapped_som, help_mapped_som, strlen(help_mapped_som) -4);
   			strcat(mapped_som, "unit.labelunit");
				
				if ((mapped_labelunit=fopen(mapped_som,"a"))==NULL)
  			{
  				fprintf(stderr,"Error opening mapped label unitfile %s",mapped_som);
  				return 0;
  			}
				else
				{
					fprintf(mapped_labelunit,"$SOMABOVE %s\n", map);
				}
 	      fclose(mapped_labelunit);
  			
   		}
 		}
 	}
	fclose(fp);
 	fclose(ofp);
 	return 1;
}
/*
##############################
int do_the_catman (unit curr_unit,FILE *opf,char map[256])
##############################
Cats given files from curr_unit.filelist[] in an outputfile
than the outputfile is copied to a already created directory tmp
with the given model from args (see arg parsing in int main()) the
java programm KEAKeyphrasEextractor
the generated keyfile is copied one directory up ../
the name of the generated keyfile is written to the FILE *opf
*/

int do_the_catman(unit curr_unit,FILE *opf,char map[256])
{
	FILE *working_file;
	FILE *output_file;
	FILE *output_keyfile;
	char working_filename[MAXFILELENGTH];
	char output_filename [MAXFILELENGTH] ="\0";
	char output_keyfilename[MAXFILELENGTH];
	char help[64];
	char c;
	char command[256];
	char javacommand[256];
	char str_number_of_phrases[4];
	char line[MAXLINELENGTH];
	// initialize char arrays
	memset(output_keyfilename,0,sizeof(output_keyfilename));
	memset(output_filename,0,sizeof(output_filename));
	memset(help,0,sizeof(help));

	strcpy(output_filename, kea_tmp_path);
	strcat(output_filename, strrchr(map, '/')+1); //extract the filename from path
//	strcat(output_filename,map);
	strcat(output_filename,".");
	strncpy(help,curr_unit.pos_x_str, strlen(curr_unit.pos_x_str) -1);
	strcat(output_filename,help);
	strcat(output_filename,"_");
	strncpy(help,curr_unit.pos_y_str, strlen(curr_unit.pos_y_str) -1);
	strcat(output_filename,help);
	strcat(output_filename,".txt");
	if ((output_file=fopen(output_filename,"w"))==0)
	{
		fprintf(stderr, "Couldn't write to disk %s\n", output_filename);
		return 0;
	}
	for (int i=0;i<curr_unit.nr_mapped_vecs;i++)
	{
		// determmine *.txt file name, open it and put the content in output_file
		strcpy(working_filename, sourcepath);
		strcat(working_filename,curr_unit.filelist[i].name);
		//strcat(working_filename, ".txt");
		//printf("file %d %s\n",i, working_filename);
		if ((working_file=fopen(working_filename, "r"))==0)
		{
			fprintf(stderr, "File not found %s\n", working_filename);
			return 0;
		}
		while ((c = fgetc(working_file)) != EOF)
		{
   		fprintf(output_file,"%c",c);		// O/P the character to outputfile
 		}
 		fclose(working_file);
 	}
 	fclose(output_file);
 	// extract keyphrases for the catted file
 	strcpy(javacommand, KEACALL);
 	strcat(javacommand, " -l ");
 	strcat(javacommand, kea_tmp_path);
 	strcat(javacommand, " -m ");
 	strcat(javacommand, model);
 	strcat(javacommand, " -n ");
 	sprintf(str_number_of_phrases,"%d",number_of_phrases);
 	strcat(javacommand, str_number_of_phrases);
 	strcat(javacommand, " -a");
 	//printf("\n%s\n", javacommand);
	system(javacommand);
	// copy catted file to directory key (perfomancereason for the next KEA call)
 	strcpy(command, "mv ");
 	strcat(command, output_filename);
 	strcat(command," ");
 	strcat(command,keypath);
 	strcat(command," -f");
	system(command);
	// read the phrases + scoring and write it to global tmp-outputfile opf

	strncpy(output_keyfilename,output_filename, strlen(output_filename) -4);

	// remove ending .txt and replace it with .key
	strcat(output_keyfilename,".key");
	if ((output_keyfile=fopen(output_keyfilename,"r"))==0)
	{
		fprintf(stderr, "Uiiii, where is it ??  /%s\n", output_keyfilename);
		return 0;
	}
	// writing rendered keyphrases to opf
	fprintf(opf, "$MODE0_PHRASES %d\n", number_of_phrases);
	while ( fgets(line, MAXLINELENGTH, output_keyfile) != NULL)
	{
		fprintf(opf, "$M0_LABEL %s", line);
	}
	fclose(output_keyfile);



 	return 1;
}

int do_the_calcman(unit curr_unit,FILE *opf)
{
	FILE *keyfile;
	int phrasecount;
	phrase phrases[MAXPHRASESPERUNIT];
	phrase helpphrase;
	char keyfilename[MAXFILELENGTH];
	char *substring;
	char keyline[MAXLINELENGTH];
	int i, whereis;
	int helpcount;
	//fprintf(stdout,"Unit %d %d\n", curr_unit.pos_x, curr_unit.pos_y);
	phrasecount = 0;
	//phrases = malloc(1 * sizeof(phrase));
	for (i=0;i<curr_unit.nr_mapped_vecs;i++)
	{
		strcpy(keyfilename,keypath);
		strcat(keyfilename,curr_unit.filelist[i].name);
		strcat(keyfilename,keyfileending);
		// cat correct filename
		//fprintf(stdout,"%s\n",keyfilename);
		if ((keyfile=fopen(keyfilename,"r"))==0)
		{
			fprintf(stderr,"Couldn't open file %s\n",keyfilename);
			return 0;
		}
		while(fgets(keyline,MAXLINELENGTH,keyfile)!=NULL)
  	{
  		// one line is read and has to be parsed
  		//phrases, stemmed phrases and score are seperated throug an
  		//horizontal tab "\t"
			substring=strtok(keyline,"\t");
			strcpy(helpphrase.phrase, substring);
			if ((substring=strtok(NULL,"\t"))==NULL)
			{
				fprintf(stderr,"Wrong format of keyfile: %s\n",keyfilename);
				fprintf(stderr,"Desired format:phrase stemmed_phrase score !!\n");
				fprintf(stderr,"Start KEAKeyphraseExtractor with option -a !");
				return 0;
			}
			strcpy(helpphrase.stemmed_phrase, substring);
			if ((substring=strtok(NULL,"\t"))==NULL)
			{
				fprintf(stderr,"Wrong format of keyfile: %s\n",keyfilename);
				fprintf(stderr,"Desired format:phrase stemmed_phrase score !\n");
				fprintf(stderr,"Start KEAKeyphraseExtractor with option -a !");
				return 0;
			}
  		helpphrase.score = atof(substring);
  		if ((whereis = phrase_exists(phrases,helpphrase,phrasecount))==-1)
  		{ // new phrase
        if (phrasecount < MAXPHRASESPERUNIT)
				{
			  	phrases[phrasecount]=helpphrase;
  				phrasecount++;
					if (phrasecount == MAXPHRASESPERUNIT)
					{  // if max is reached, try to reduce the phrasecount through
						// filtering the phrases
							phrasecount = filter_phrases(phrases, phrasecount);
					}
				}
				else
				{	 // max is reached phrase can't be computed	
        	fprintf(stdout,"%s omitted, maximum phrases per unit (%d) reached!!\n" , helpphrase.phrase, MAXPHRASESPERUNIT);
 	  		}
			}
  		else
  		{
  			phrases[whereis].score = phrases[whereis].score + helpphrase.score;
  		}
  	}
  	fclose(keyfile);
  }
	// and now sort the array
  phrasecount = filter_phrases(phrases, phrasecount);
  phrasecount = filter_phrases(phrases, phrasecount);
  phrasecount = filter_phrases(phrases, phrasecount);
	sort_array(phrases, phrasecount);
//	fprintf(stdout,"New Unit\n");
// 	for (int g=0; g<phrasecount;g++)
//	{
//		fprintf(stdout, "%s\t%s\t%f\n",phrases[(phrasecount-1)-g].phrase,phrases[(phrasecount-1)-g].stemmed_phrase,phrases[(phrasecount-1)-g].score);
//	}

	
 	// write results to new unitfile opf
 	int g;
 	//check , if there are enough phrases extracted
 	// and write token into new unitfile opf
//	fprintf(stdout, "new unit %d\n", phrasecount);
 	if (phrasecount > number_of_phrases)
 	{
 	 	helpcount = number_of_phrases;
 		while ((phrasecount > helpcount) && (phrases[(phrasecount) - helpcount].score == phrases[(phrasecount) - helpcount-1].score))
	 	{
		  //fprintf(stdout,"phr:%s score(%d) %f phr:%s score(%d) %f\n", phrases[helpcount].phrase, helpcount, phrases[helpcount].score , phrases[helpcount-1].phrase, helpcount-1, phrases[helpcount-1].score);
 			helpcount++;
	 	}
	}
	else
	{
		helpcount = phrasecount;
	}
	fprintf(opf,"$MODE1_PHRASES %d\n", helpcount);
 	
 	for (g=0;g<helpcount;g++)
	{
		fprintf(opf, "%s\t%s\t%f\n",phrases[(phrasecount-1)-g].phrase,phrases[(phrasecount-1)-g].stemmed_phrase,phrases[(phrasecount-1)-g].score);
	}
	return 1;
}

// sum_phrase : checks if a given phrase current_phrase is a subphrase of the phrase array
// phrases (the whole array is checked). if it is accasionally a subphrase, the score of that
// array element's score is increased by the current_phrase score. is_subphrase is set to true
// at the end is_subphrase is returned
int sum_phrases(phrase phrases[MAXPHRASESPERUNIT], int phrasecount, phrase current_phrase)
{
	int is_subphrase;
  is_subphrase = FALSE;

	for (int j=0;j < phrasecount; j++)
 	{
 		if (strlen(current_phrase.stemmed_phrase) < strlen(phrases[j].stemmed_phrase))
 		{ // compare if one stemmed phrase is substring of the other
 			if (compare_phrases(phrases[j],current_phrase) == TRUE)
 			{ // if so
 					phrases[j].score = phrases[j].score + current_phrase.score;
 				  is_subphrase = TRUE;  // current_phrase has to be disposed later on
 			}
 		}
	}
	return is_subphrase;
}

int filter_phrases(phrase phrases[MAXPHRASESPERUNIT], int phrasecount)
{
	phrase help_phrase;
	int filtered_counter = 0;
	int new_phrase;
	char *help;
	for (int i=0; i < phrasecount; i++)
	{
		if (sum_phrases(phrases, phrasecount, phrases[i]) == TRUE)
		{
			dispose_phrase(phrases,i, phrasecount);
			i--;
			phrasecount--;
		
		}
	}
//	fprintf (stdout,"\nNew unit ... \n");
//	for (int xx =0; xx < phrasecount; xx++)
//	{
//		fprintf(stdout,"Phrase %s\n",phrases[xx].phrase);
//	}
	return phrasecount;
}

//####################################################################################
// Compare phrases compares phrase 1 and 2 and return TRUE if phrase2 is a subphrase from 1
//#####################################################################################
int compare_phrases(phrase phrase1, phrase phrase2)
{
	char *help1;
	char *help2;

	if ((help1=strstr(phrase1.stemmed_phrase,phrase2.stemmed_phrase)) != NULL)
	{
//  	fprintf (stdout, "comparing %s with %s\n",phrase1.stemmed_phrase,phrase2.stemmed_phrase);
		help2 = help1;
		help1 = help1 + strlen(phrase2.stemmed_phrase);
		if ((*help1 != ' ') && (*help1 != '\0'))
		{
			return FALSE;
		}
		else if((strncmp(phrase1.stemmed_phrase,phrase2.stemmed_phrase,strlen(phrase2.stemmed_phrase))) == 0)
		{ // substring is at begin of phrase so i don't have to check if there is a space before
			return TRUE;
		}
		else
		{ // check if there is a space before the word
			help2--;
			if (*help2 == ' ')
			{
				return TRUE;
			}
		}
	}
	return FALSE;
}
//#####################################################################################
// dispose_phrase: the element [position] in the phrasearray is disposed
// the following elements are are copied one position up
//#####################################################################################
void dispose_phrase(phrase phrases[MAXPHRASESPERUNIT],int position, int phrasecount)
{
	for (int yy = position; yy < phrasecount; yy++)
	{
		phrases[yy] = phrases[yy+1];
	}
}
// ####################################################################################
// phrase_exists: checks if a given phrase current_phrase exists in the phrase
// array extracted_phrases and returns the position
// otherwise the functions returns -1
// ####################################################################################
int phrase_exists(phrase extracted_phrases[MAXPHRASESPERUNIT], phrase current_phrase, int dim)
{
 	int i;
 	for(i=0 ;i<dim;i++)
	{
 	 	if (strcmp(extracted_phrases[i].stemmed_phrase,current_phrase.stemmed_phrase)==0)
 		{
 			return i;
 		}
 	}
	return -1;
}
// ################################################################
// THe function opens the tmpfile, serches for the token $MODE1
// and reads the phrases + score in an phrase array
// then the phrases are sorted by the score and writen in an
// keyfile "map.key"
// ################################################################
int write_map_keyfile(char map[MAXFILELENGTH])
{
	FILE *unitlabelfile;
	FILE *keyfile;
	char line[MAXLINELENGTH];
	// char c;
	// int i;
	char keyfilename[MAXFILELENGTH];
	char keyfilepath[MAXFILELENGTH];
	char unitlabelname[MAXFILELENGTH];
	int phrase_linecount;
	char *token;
	char *substring;
	phrase phrases[MAXPHRASESPERUNIT];
	phrase helpphrase;
	int phrasecount;
	int whereis;
	int helpcount;

	// open labelunit file
	phrasecount = 0;
	strcpy(unitlabelname, map);
	strcat(unitlabelname,".labelunit"); // write to new unit file .labelunit
	if ((unitlabelfile = fopen(unitlabelname,"r"))==0)
	{
		fprintf(stderr,"Couldn't open labelunit file %s !\n",unitlabelname);
		return 0;
	}

	// open file for writing key-phrases for map
	// 1st: set the right path

	strcpy(keyfilepath, keypath);
	strcpy(keyfilename, strrchr(map,'/')+1); //extract the filename from path
	strcat(keyfilepath, keyfilename);
	strcat(keyfilepath, keyfileending);
	// results in keyfilepath/unitfilename.keyfileending
	// e.g. output/key/democoll_1_1_0_0.unit.key

	if ((keyfile = fopen(keyfilepath,"w"))==0)
	{
		fprintf(stderr,"Couldn't write keyfile %s !\n",keyfilepath);
		return 0;
	}


	//line = (char *) malloc(1 * sizeof(char));
	//memset(line,0, strlen(line) * sizeof(char));
	phrase_linecount = 0;
  while (fgets(line,MAXLINELENGTH,unitlabelfile)!=NULL)
  {
// 		line = (char *) malloc(1 * sizeof(char));
//  	//line = (char *) realloc(line,1 * sizeof(char));
//   	i=0;
//  	do
//  	{
//  		line = (char *) realloc(line,(i+2) * sizeof (char));
//  		// +2 because to reserve space for '\0'
//  		line[i] = c;
//  		i++;
//  	}while ((c=fgetc(unitlabelfile)) != '\n');
//  	line[i] = '\0';
//		fprintf (stderr,"%s\n", line);
 		if (phrase_linecount > 0)
 		{
			// read phrase_linecount lines and extract phrases
			substring=strtok(line,"\t");
			strcpy(helpphrase.phrase, substring);
			if ((substring=strtok(NULL,"\t"))==NULL)
			{
				fprintf(stderr,"Wrong format of keyfile: %s\n",keyfilename);
				fprintf(stderr,"Desired format:phrase stemmed_phrase score !!\n");
				fprintf(stderr,"Start KEAKeyphraseExtractor with option -a !");
				return 0;
			}
			strcpy(helpphrase.stemmed_phrase, substring);
			if ((substring=strtok(NULL,"\t"))==NULL)
			{
				fprintf(stderr,"Wrong format of keyfile: %s\n",keyfilename);
				fprintf(stderr,"Desired format:phrase stemmed_phrase score !\n");
				fprintf(stderr,"Start KEAKeyphraseExtractor with option -a !");
				return 0;
			}
	 		helpphrase.score = atof(substring);
  		if ((whereis = phrase_exists(phrases,helpphrase,phrasecount))==-1)
  		{ // new phrase
        if (phrasecount < MAXPHRASESPERUNIT)
				{
  				phrases[phrasecount]=helpphrase;
  				phrasecount++;
					if (phrasecount == MAXPHRASESPERUNIT)
					{  // if max is reached, try to reduce the phrasecount through
						// filtering the phrases
							phrasecount = filter_phrases(phrases, phrasecount);
					}
				}
				else
				{	 // max is reached phrase can't be computed	
        	fprintf(stdout,"phrase %s is omitted, maximum phrases per unit (%d) reached!!\n" , helpphrase.phrase, MAXPHRASESPERUNIT);

 	  		}
			}
  		else
  		{
  			phrases[whereis].score = phrases[whereis].score + helpphrase.score;
  		}
  		phrase_linecount--;
 		}
 		else
 		{
 			token=strtok(line, " ");
 			if (strcmp(token,"$MODE1_PHRASES")==0)
 			{
 				substring=strtok(NULL," ");
 				phrase_linecount=atoi(substring);

	 		}
	 	}
 	}
int g;
  // debug info

	phrasecount = filter_phrases(phrases, phrasecount);
	phrasecount = filter_phrases(phrases, phrasecount);
	phrasecount = filter_phrases(phrases, phrasecount);
 	// sort array
 	sort_array(phrases, phrasecount);

 	
 	if (phrasecount > number_of_phrases)
 	{
 	 	helpcount = number_of_phrases;
 		while ((phrasecount > helpcount) && (phrases[(phrasecount) - helpcount].score == phrases[(phrasecount) - helpcount-1].score))
	 	{
 			//fprintf(stdout,"phr:%s score(%d) %f phr:%s score(%d) %f\n", phrases[(phrasecount) - helpcount].phrase, helpcount, phrases[(phrasecount) - helpcount].score , phrases[(phrasecount) - helpcount-1].phrase, helpcount-1, phrases[(phrasecount) - helpcount].score);
 			helpcount++;
	 	}
	}
	else
	{
		helpcount = phrasecount;
	}
	fprintf(keyfile,"$MODE1_PHRASES %d\n", helpcount);
 	//check , if there are enough phrases extracted
 	// and write token into new unitfile opf
 	for (g=0; g<helpcount;g++)
	{
		fprintf(keyfile, "%s\t%s\t%f\n",phrases[(phrasecount-1)-g].phrase,phrases[(phrasecount-1)-g].stemmed_phrase,phrases[(phrasecount-1)-g].score);
	}
 	fclose(unitlabelfile);
 	fclose(keyfile);
	return 1;
}


// ############################################################
// Sorts (ascending) an array of phrases by the field score
// using bubble sort
// ############################################################
void sort_array(phrase phrases[MAXPHRASESPERUNIT], int phrasecount)
{
	phrase helpphrase;
	
	for (int i=0;i<phrasecount;i++)
 	{
 		for(int j=phrasecount-1;j>i;j--)
 		{
 			if(phrases[j-1].score > phrases[j].score)
 			{
 				helpphrase=phrases[j-1];
 				phrases[j-1]=phrases[j];
 				phrases[j]=helpphrase;
 			}
 		}
 	}
}

//int write_labels(FILE *input, FILE *output)
//{
//	char c;
//	char *labelline;
//	char *substring;
//	int i;	
//
//	if ((c=fgetc(input)) != EOF)
//	{
//		labelline = (char *) malloc(1 * sizeof(char));
//  	memset(labelline,0, strlen(labelline) * sizeof(char));	
// 		i=0;
// 		do
//		{
// 			labelline = (char *) realloc(labelline,(i+2) * sizeof (char));
// 			// +2 because to reserve space for '\0'
// 			labelline[i] = c;
// 			i++;
// 		}while ((c=fgetc(input)) != '\n');
//		labelline[i] = '\0';
// 		if ((strcmp(labelline, "LEEREUNIT")) != 0)
// 		{
//			if (strlen(labelline) < 1)
//			{
//				fprintf(stderr, "Shit !!\n");
//				return 0;
//			}
//			else
//			{
//				substring=strtok(labelline, " ");
//				fprintf(output,"$LABELSMMX %s", substring);
//		 		while(((substring=strtok(NULL, " ")) != NULL) && (strcmp(substring, "LABELSENDE")!=0))
//		 		{
//		 			fprintf(output, " %s", substring);
//		 		}
//		 		fprintf(output, "\n");
//			}
// 		}
//	free(labelline);
// 	}
// 	else
// 	{
// 		fprintf(stderr, "Error in Labelfile, no more lines although expected\n");
// 		return 0;
// 	}
//	return 1;
//}

int is_splitted_file(char lastfilename[MAXFILELENGTH],char newfilename[MAXFILELENGTH])
{
	FILE *lastfile;
	FILE *newfile;
	char pathnamelast[MAXFILELENGTH];
	char pathnamenew[MAXFILELENGTH];
	
	
	strcpy(pathnamelast, sourcepath);
	strcat(pathnamelast,lastfilename);
	if ((lastfile=fopen(pathnamelast,"r"))==NULL)
	{
		return 1;
	}
	else
	{
		fclose(lastfile);
	}
	strcpy(pathnamenew, sourcepath);
	strcat(pathnamenew,newfilename);
	if ((newfile=fopen(pathnamenew,"r"))==NULL)
	{
		return 1;
	}
	else
	{
		fclose(newfile);
	}
	return 0;
}


